@@ -5,6 +5,7 @@ import sqlite3
 
 from .parser import parse_fasta
 
+
 class Preprocessor(object):
   '''
   Object class that takes in a proteome FASTA file, k for k-mer size (split), and format to 
@@ -20,7 +21,7 @@ class Preprocessor(object):
   Optional: protein IDs can be versioned, so the versioned_ids argument can be passed
   as True to store them as versioned.
   '''
-  def __init__(self, proteome, split, preprocess_format, database='', smaller_proteome='', versioned_ids = False):
+  def __init__(self, proteome, split, preprocess_format, database='', one_gene_proteome='', versioned_ids = False):
     if split < 2:
       raise ValueError('k-sized split is invalid. Cannot be less than 2.')
 
@@ -31,7 +32,7 @@ class Preprocessor(object):
     self.split = split
     self.preprocess_format = preprocess_format
     self.database = database
-    self.smaller_proteome = smaller_proteome
+    self.one_gene_proteome = one_gene_proteome
     self.versioned_ids = versioned_ids
 
   def split_protein(self, seq, k):
@@ -52,7 +53,7 @@ class Preprocessor(object):
     for being able to load the data in when a query is called.
     '''
     name = self.proteome.split('/')[-1].split('.')[0]
-    with open(name + '_kmers' + '_' + str(self.split) + '.pickle', 'wb') as f:
+    with open(name + '_' + str(self.split) + 'mers' + '.pickle', 'wb') as f:
       pickle.dump(kmer_dict, f)
     with open(name + '_names.pickle', 'wb') as f:
       pickle.dump(names_dict, f)
@@ -64,22 +65,14 @@ class Preprocessor(object):
     for searching. This is much faster for exact matching.
     '''
     name = self.proteome.split('/')[-1].split('.')[0]
-    kmers_table = name + '_kmers' + '_' + str(self.split)
+    kmers_table = name + '_' + str(self.split) + 'mers'
     names_table = name + '_names'
 
-    if self.smaller_proteome != '':
-      smaller_proteome_ids = []
-      smaller_proteome = parse_fasta(self.smaller_proteome)
-      for protein in smaller_proteome:
-        protein_id = protein.id.split('|')[1]
-        if self.versioned_ids:
-          protein_id += '.' + str(protein.description).split('SV=')[1][0]
-        smaller_proteome_ids.append(protein_id)
-
     conn = sqlite3.connect(self.database)
     c = conn.cursor()
+
     c.execute('CREATE TABLE IF NOT EXISTS "{k}"(kmer TEXT, position INT)'.format(k = kmers_table))
-    c.execute('CREATE TABLE IF NOT EXISTS "{n}"(protein_number INT, protein_id TEXT, in_smaller_proteome INT, protein_existence_level INT)'.format(n = names_table))
+    c.execute('CREATE TABLE IF NOT EXISTS "{n}"(protein_number INT, protein_id TEXT, pe_level INT, gene_priority INT)'.format(n = names_table))
 
     # make a row for each unique k-mer and position mapping
     for kmer, positions in kmer_dict.items():
@@ -88,11 +81,8 @@ class Preprocessor(object):
 
     # make a row for each number to protein ID mapping
     for protein_number, protein_data in names_dict.items():
-      
-      in_smaller_proteome = 1 if protein_data[0] in smaller_proteome_ids else 0
-      
-      c.execute('INSERT INTO "{n}"(protein_number, protein_id, in_smaller_proteome, protein_existence_level) VALUES(?, ?, ?, ?)'.format(n = names_table), 
-        (protein_number, protein_data[0], in_smaller_proteome, protein_data[1]))
+      c.execute('INSERT INTO "{n}"(protein_number, protein_id, pe_level, gene_priority) VALUES(?, ?, ?, ?)'.format(n = names_table), 
+        (protein_number, protein_data[0], protein_data[1], protein_data[2]))
 
     # create indexes for both k-mer and name tables
     c.execute('CREATE INDEX IF NOT EXISTS "{id}" ON "{k}"(kmer)'.format(id = kmers_table + '_id', k = kmers_table))
@@ -115,6 +105,13 @@ class Preprocessor(object):
     names_dict = {}
     protein_count = 1
 
+    if self.one_gene_proteome != '':
+      one_gene_proteome_ids = []
+      one_gene_proteome = parse_fasta(self.one_gene_proteome)
+      for protein in one_gene_proteome:
+        protein_id = protein.id.split('|')[1]
+        one_gene_proteome_ids.append(protein_id)
+
     for protein in proteome:
       kmers = self.split_protein(str(protein.seq), self.split)
       for i in range(len(kmers)):
@@ -124,15 +121,24 @@ class Preprocessor(object):
           kmer_dict[kmers[i]] = [protein_count * 100000 + i]     # create entry for new k-mer
 
       # create names mapping # to protein ID (include versioned if argument is passed) 
-      protein_id = str(protein.description).split(' ')[0]
       try:
-        protein_existence_level = int(str(protein.description).split('PE=')[1][0])
-        if self.versioned_ids:
-          names_dict[protein_count] = (protein_id.split('|')[1] + '.' + str(protein.description).split('SV=')[1][0], protein_existence_level)
-        else:
-          names_dict[protein_count] = (protein_id.split('|')[1], protein_existence_level)
+        protein_id = protein.id.split('|')[1]
       except IndexError:
-        names_dict[protein_count] = str(protein.description).split(' ')[0]
+        protein_id = protein.id
+
+      if self.one_gene_proteome != '':
+        gene_priority = 1 if protein_id in one_gene_proteome_ids else 0
+      else:
+        gene_priority = None
+
+      try:
+        pe_level = int(str(protein.description).split('PE=')[1][0])
+        if self.versioned_ids:
+          names_dict[protein_count] = (protein_id + '.' + str(protein.description).split('SV=')[1][0], pe_level, gene_priority)
+        else:
+          names_dict[protein_count] = (protein_id, pe_level, gene_priority)
+      except IndexError:
+        names_dict[protein_count] = (str(protein.description).split(' ')[0], None, None)
 
       protein_count += 1
 
